import random
import os.path
import time
import re
import json

POR_QUE_NACI_IMG = "{}/por-que-naci.jpg".format(os.path.dirname(__file__))
MISERIBANK_DATA = "{}/miseribank.json".format(os.path.dirname(__file__))

handlers = []

def register(regex, ignore_case=False):
    flags = [re.IGNORECASE] if ignore_case else []
    compiled_regex = re.compile(regex, *flags)

    def wrap(handler):
        handlers.append((compiled_regex, handler))
        return handler

    return wrap


def handle(event, state):
    print(repr(event.msg.__dict__))
    text = event.msg.content

    for regex, handler in handlers:
        match = re.search(regex, text)
        if match:
            handler(event, *match.groups(), **match.groupdict())
            break

@register(r'^Satanás\?$')
def miau(event):
    event.msg.chat.sendMsg("Miau!")

@register(r'^Quem está aí\?$')
def outro_gato(event):
    event.msg.chat.sendMsg("Outro gato!")


@register(r'¡ay!', ignore_case=True)
def ay(event):
    event.msg.chat.sendFile(open(POR_QUE_NACI_IMG, "rb"), "pq.jpg", image=True)

@register(r'já dá pra ir pra casa\?', ignore_case=True)
def ir_pra_casa(event):
    event.msg.chat.sendMsg('pra fazer nada não precisa estar remoto')

@register('^faz parte do meu estilo$', ignore_case=True)
def estilo(event):
    event.msg.chat.sendMsg('legal')
    time.sleep(2)
    event.msg.chat.sendMsg('você tira onda')

@register(r'^hm!$', ignore_case=True)
def hm(event):
    event.msg.chat.sendMsg('cara... você é foda')
    time.sleep(2)
    event.msg.chat.sendMsg('sério mesmo, cara, você é muito foda')
    time.sleep(1)
    event.msg.chat.sendMsg('você é demais!')

@register('^você tira onda$', ignore_case=True)
def voce_tira_onda(event):
    event.msg.chat.sendMsg('hm!')

@register('^Xeroque$')
def xeroque(event):
    event.msg.chat.sendMsg('upupup')
    event.msg.chat.sendMsg('upupup')

@register('^pkill -9 bot$')
def pkill(event):
    event.msg.chat.sendMsg('!![1] Killed       python elmord_bot.py')


##### MiseriBank 3000 #####
def remove_diacritics(text):
    from unicodedata import normalize
    return ''.join(normalize('NFD', char)[0] for char in text)

def normalize_name(text):
    return remove_diacritics(text).capitalize()

def load_miseribank_data():
    with open(MISERIBANK_DATA, 'r') as f:
        return json.load(f)

def save_miseribank_data(data):
    with open(MISERIBANK_DATA, 'w') as f:
        return json.dump(data, f)

@register(r'([\w0-9\[\]]+)\s+([+-])\s*([0-9]+(?:[.,][0-9]+)?)\s*MSR$')
def miseribank_add(event, person, sign, magnitude):
    name = normalize_name(person)
    value = float(sign + magnitude.replace(',', '.'))
    data = load_miseribank_data()
    if name in data:
        data[name] += value
        save_miseribank_data(data)
        event.msg.chat.sendMsg("{}: {:.2f} MSR ({:+.2f})".format(name, data[name], value))
    else:
        event.msg.chat.sendMsg("Quem é {}?".format(name))

@register(r'^MSR$')
def miseribank_status(event):
    data = sorted(load_miseribank_data().items(),
                  key=lambda item: (-item[1], item[0]))
    output_format = '{:12} |   {}\n'
    output = 'MiseriBank™ 3000\n'
    for name, value in data:
        formatted_value = '{:.2f}'.format(value)
        output += output_format.format(name, formatted_value)

    event.msg.chat.sendMsg('{code}\n' + output + '{code}')

@register('^MSR ☭$')
def miseribank_redistribute(event):
    data = load_miseribank_data()
    redistributed_value = sum(data.values()) / len(data)
    for name in data:
        data[name] = redistributed_value
    save_miseribank_data(data)
    event.msg.chat.sendMsg("All power to the proletariat!")


@register('^#elf$')
def elvish(event):
    event.msg.chat.sendMsg("Elen síla lúmenn' omentielvo")

@register('^#moeda$')
def elvish(event):
    event.msg.chat.sendMsg(random.choice(["cara", "coroa"]))

def enable_filho_do_capeta():
    filho_do_capeta = [
        (r'Porra é o cu da cachorra!', 1),
        (r'A cachorra morreu, teu cu é meu!', 1),
        (r'Ah, então... meu cu é seu? Meu cu é teu, então... Tá insinuando que eu sou viado?', 1),
        (r'Sim! Tu é viado!', 1),
        (r'Viado é rolimã, como tu e tua irmã!', 1),
        (r'Viado é borboleta, como teu pai pelo c* e tua mãe pela b*****!', 1),
        (r'Ah, vai tomá no cu!', 1),
        (r'Tomate cru é vitamina, como tu e tua prima!', 1),
        (r'Tomate cru pra salada, e teu cu pra moçada!', 5),
        (r'FILHO DA PUTA!!!', 1),
    ]

    def add_reply(question, answer, delay):
        def handle(event):
            event.msg.chat.setTyping()
            time.sleep(delay)
            event.msg.chat.sendMsg(answer)
            event.msg.chat.setTyping(False)
        handlers.append((re.compile('^{}$'.format(re.escape(question))), handle))

    for i in range(len(filho_do_capeta)-1):
        question, delay = filho_do_capeta[i]
        answer, _ = filho_do_capeta[i+1]
        add_reply(question, answer, delay)



# @register(r'')
# def ou_nao(event):
#     if random.randrange(8) == 0:
#         event.msg.chat.sendMsg('ou não')
